/** The abstract superclass of all AST nodes */

package AST;

import Typechecker.*;
import Helpers.*;
import java.util.*;

public abstract class ASTNode {
    // Instance variables

    // The line number where the node is in the source file, for use
    // in printing error messages about this AST node
    protected int line_number;


    // Constructor
    public ASTNode(int ln) {
	line_number = ln;
    }


    // AST printing

    /** return a printable representation of this ast node.  the
     * result should *not* end in a newline. **/
    public String toString() {
	return toString("");
    }

    /** return a printable representation of this ast node, with all
     * lines after the first prefixed by indent.  the result should
     * *not* end in a newline. **/
    // subclasses override this to provide their pretty-printing behavior
    public abstract String toString(String indent);


    // HELPERS

    /** return a printable representation of the argument sequence of
     * AST nodes, with the given indent, separated by the given
     * separator string */
    protected static <Node extends ASTNode> String separatedListToString(
		List<Node> asts, String indent, String separator) {
	String result = "";
	Iterator<Node> iter = asts.iterator();
	boolean first = true;
	while (iter.hasNext()) {
	    Node ast = iter.next();
	    if (first) {
		first = false;
	    } else {
		result += separator;
	    }
	    result += ast.toString(indent);
	}
	return result;
    }

    /** return a printable representation of the argument sequence of
     * AST nodes, with the given indent, each element prefixed by
     * the given prefix string */
    protected static <Node extends ASTNode> String prefixedListToString(
		List<Node> asts, String indent, String prefix) {
	String result = "";
	Iterator<Node> iter = asts.iterator();
	while (iter.hasNext()) {
	    Node ast = iter.next();
	    result += prefix + ast.toString(indent);
	}
	return result;
    }

    /** return a printable representation of the argument sequence of
     * AST nodes, with the given indent, each element prefixed by the
     * given prefix string and terminated by the given terminator
     * string */
    protected static <Node extends ASTNode> String
		prefixedAndTerminatedListToString(List<Node> asts, 
						  String indent, 
						  String prefix, 
						  String terminator) {
	String result = "";
	Iterator<Node> iter = asts.iterator();
	while (iter.hasNext()) {
	    Node ast = iter.next();
	    result += prefix + ast.toString(indent) + terminator;
	}
	return result;
    }

    /** return the indentation string to use for an additional level
     * of indentation beyond the argument */
    public static String nested_indent(String indent) {
	return indent + "  ";
    }
}
