/** The abstract superclass of all unary operator expression AST
 * nodes that are for numeric arithmetic operations, e.g., -. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public abstract class ArithmeticUnopExpr extends UnopExpr {
    // Constructor
    public ArithmeticUnopExpr(Expr a, int ln) {
	super(a, ln);
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// typecheck the argument
	ResolvedType arg_type = arg.typecheck(st);

	// verify that it's an int
	arg_type.checkIsIntType("argument", line_number);

	// yields the same as the argument type
	return arg_type;
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// evaluate the argument
	Value arg_value = arg.evaluate(env);

	// compute & return the result
	return new IntValue(
	    evaluateIntOperation(
		arg_value.getIntValue(line_number)));
    }

    /** Evaluate the integer version of this operation */
    protected abstract int evaluateIntOperation(int arg_value);


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// construct the argument subexpression
	ILExpr arg_expr = arg.lower(il_fun_decl);

	// return an IL expression for computing the result
	return loweredIntOperation(arg_expr);
    }

    /** Return the integer version of this operation */
    protected abstract ILExpr loweredIntOperation(ILExpr arg_expr);
}
