/** The AST node for an assignment statement. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class AssignStmt extends Stmt {
    // Instance variables

    // AST subtrees:
    protected AssignableExpr lhs;
    protected Expr rhs;


    // Constructor
    public AssignStmt(AssignableExpr l, Expr r, int ln) {
	super(ln);
	lhs = l;
	rhs = r;
    }


    // AST printing
    public String toString(String indent) {
	return lhs.toString(indent) + " = " + rhs.toString(indent) + ";";
    }


    // Typechecking operations

    /** Typecheck this statement in the given enclosing context */
    public void typecheck(CodeSymbolTable st)
	    throws TypecheckCompilerException {
	// typecheck the lhs expression
	ResolvedType lhs_type = lhs.typecheck(st);

	// typecheck the rhs expression
	ResolvedType rhs_type = rhs.typecheck(st);

	// verify that the type of the rhs can be assigned to the type
	// of the lhs
	rhs_type.checkIsAssignableTo(lhs_type, "rhs", "lhs", line_number);
    }


    // Evaluation operations

    /** Evaluate this statement in the given environment */
    public void evaluate(CodeEnvironment env) throws EvalCompilerException {
	// ask the lhs expression to evaluate the assignment
	lhs.evaluateAssignment(env, rhs);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code
     * for this statement */
    public void lower(ILFunDecl il_fun_decl) {
	// ask the lhs expression to evaluate the assignment
	lhs.lowerAssignment(il_fun_decl, rhs);
    }
}
