/** The abstract superclass of all expression AST nodes that can also
 * appear on the left-hand-side of an assignment statement, i.e., can
 * be l-values. */

package AST;

import Evaluator.*;
import IL.*;
import Helpers.*;

public abstract class AssignableExpr extends Expr {
    // Constructor
    public AssignableExpr(int ln) {
	super(ln);
    }


    // Evaluation operations

    /** Evaluate an assignment to this expression with the given rhs
     * expression in the given environment */
    public abstract void evaluateAssignment(CodeEnvironment env, 
					    Expr rhs) 
	throws EvalCompilerException;


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code to
     * assign to this expression the given rhs expression */
    public abstract void lowerAssignment(ILFunDecl il_fun_decl, Expr rhs);
}
