/** The AST node for a nested block of statements. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;
import java.util.*;

public class BlockStmt extends Stmt {
    // Instance variables

    // AST subtrees:
    protected List<Stmt> stmts;


    // Constructor
    public BlockStmt(List<Stmt> ss, int ln) {
	super(ln);
	stmts = ss;
    }


    // AST printing
    public String toString(String indent) {
	String nested_indent = nested_indent(indent);
	return
	    "{" + 
	    prefixedListToString(stmts, nested_indent, "\n" + nested_indent) +
	    "\n" + indent + "}";
    }


    // Typechecking operations

    /** Typecheck this statement in the given enclosing context */
    public void typecheck(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// create a new nested scope for the body of the block
	CodeSymbolTable nested_st = new CodeSymbolTable(st);

	// typecheck the statements in the block
	Iterator<Stmt> iter = stmts.iterator();
	while (iter.hasNext()) {
	    Stmt stmt = iter.next();
	    stmt.typecheck(nested_st);
	}
    }


    // Evaluation operations

    /** Evaluate this statement in the given environment */
    public void evaluate(CodeEnvironment env) throws EvalCompilerException {
	// create a new nested environment for the body of the block
	CodeEnvironment nested_env = new CodeEnvironment(env);

	// evaluate the statements in the block
	Iterator<Stmt> iter = stmts.iterator();
	while (iter.hasNext()) {
	    Stmt stmt = iter.next();
	    stmt.evaluate(nested_env);
	}
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code
     * for this statement */
    public void lower(ILFunDecl il_fun_decl) {
	// generate code for the statements in the nested block
	Iterator<Stmt> iter = stmts.iterator();
	while (iter.hasNext()) {
	    Stmt stmt = iter.next();
	    stmt.lower(il_fun_decl);
	}
    }
}
