/** The abstract superclass of all binary operator expression AST
 * nodes that are for boolean operations, e.g., && and ||. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public abstract class BooleanBinopExpr extends BinopExpr {
    // Constructor
    public BooleanBinopExpr(Expr a1, Expr a2, int ln) {
	super(a1, a2, ln);
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// typecheck the arguments
	ResolvedType arg1_type = arg1.typecheck(st);
	ResolvedType arg2_type = arg2.typecheck(st);

	// verify that they're booleans
	arg1_type.checkIsBooleanType("first argument", line_number);
	arg2_type.checkIsBooleanType("second argument", line_number);

	// yields a boolean
	return new ResolvedBooleanType();
    }
}
