/** The AST node for a boolean literal (a.k.a. constant) expression,
 * i.e., true or false. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class BooleanLiteralExpr extends Expr {
    // Instance variables

    // AST subtrees:
    protected boolean value;


    // Constructor
    public BooleanLiteralExpr(boolean v, int ln) {
	super(ln);
	value = v;
    }


    // AST printing
    public String toString(String indent) {
	return value ? "true" : "false";
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// yields a boolean
	return new ResolvedBooleanType();
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// return the boolean value
	return new BooleanValue(value);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// booleans are represented by integers in the IL, where 0 is
	// false and anything non-zero (e.g. 1) is true
	int il_value = value ? 1 : 0;

	// return the result value
	return new ILIntConstantExpr(il_value);
    }
}
