/** The abstract superclass of all unary operator expression AST
 * nodes that are for boolean operations, e.g., ! (not). */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public abstract class BooleanUnopExpr extends UnopExpr {
    // Constructor
    public BooleanUnopExpr(Expr a, int ln) {
	super(a, ln);
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// typecheck the argument
	ResolvedType arg_type = arg.typecheck(st);

	// verify that it's a boolean
	arg_type.checkIsBooleanType("argument", line_number);

	// yields a boolean
	return new ResolvedBooleanType();
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// evaluate the argument
	Value arg_value = arg.evaluate(env);

	// compute & return the result
	return new BooleanValue(
	    evaluateBooleanOperation(
	        arg_value.getBooleanValue(line_number)));
    }

    /** Evaluate this operation */
    protected abstract boolean evaluateBooleanOperation(boolean arg_value);


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// construct the argument subexpression
	ILExpr arg_expr = arg.lower(il_fun_decl);

	// return an IL expression for computing the result
	return loweredBooleanOperation(arg_expr);
    }

    /** Return this operation */
    protected abstract ILExpr loweredBooleanOperation(ILExpr arg_expr);
}
