/** The abstract superclass of AST nodes for class declarations */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;
import java.util.*;

public abstract class ClassDecl extends ASTNode {
    // Instance variables

    // AST subtrees:
    protected String class_name;

    // The symbol table for the scope associated with the body of this
    // class declaration
    protected ClassSymbolTable symbol_table;


    // Constructor
    public ClassDecl(String cn, int ln) {
	super(ln);
	class_name = cn;
    }


    // Typechecking operations

    /** A helper that just declares this class decl. implemented by
     * subclasses. */
    public abstract void declareClass(GlobalSymbolTable st)
	throws TypecheckCompilerException;

    /** A helper that checks and fills in the class's interface, given
     * that all classes in the program have been declared. implemented
     * by subclasses. */
    public abstract void computeClassInterface()
	throws TypecheckCompilerException;

    /** A helper that finishes typechecking of the body of the class
     * declaration, once the full class interfaces of all the
     * program's classes have been constructed. implemented by
     * subclasses. */
    public abstract void typecheckClass() throws TypecheckCompilerException;


    // Evaluation operations

    /** Evaluate the declaration part of the class. */
    public abstract void evaluateDecl(GlobalEnvironment env) 
	throws EvalCompilerException;


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language program with declarations for
     * this class */
    public abstract void lower(ILProgram il_program);

    /** A helper to declare the class's vtbl in initialized global data */
    protected void declareVtbl(ILProgram il_program) {
	String vtbl_name = class_name + ".vtbl";
	ILType vtbl_type = symbol_table.vtblRecordType();
	ILValue vtbl_value = symbol_table.vtblRecordValue();
	il_program.addGlobalVarDecl(
	    new ILInitializedGlobalVarDecl(vtbl_type, vtbl_name, vtbl_value));
    }
}
