/** The AST node for an instance variable declaration in a class */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class InstanceVarDecl extends ClassVarDecl {
    // Constructor
    public InstanceVarDecl(Type t, String n, int ln) {
	super(t, n, ln);
    }


    // Typechecking operations

    /** declare this class variable in the class symbol table */
    public void computeInterface(ClassSymbolTable st) 
		throws TypecheckCompilerException {
	// resolve the type of this variable
	ResolvedType resolved_type = type.resolve(st);

	// declare it in the class's symbol table
	st.declareInstanceVariable(resolved_type, name, line_number);
    }


    // Evaluation operations

    /** declare this class variable in the class environment */
    public void declareVariable(ClassEnvironment env) {
	// nothing to do for an instance variable; instance variables
	// are stored in each instance, not the class
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language program with whatever
     * declarations are needed for this class variable, where the
     * symbol table is for the enclosing class. */
    public void lower(ILProgram il_program, ClassSymbolTable class_st) {
	// nothing to do; instance variables are stored in each
	// instance, not the class
    }
}
