/** The AST node for an int literal (a.k.a. constant) expression. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class IntLiteralExpr extends Expr {
    // Instance variables

    // AST subtrees:
    protected int value;


    // Constructor
    public IntLiteralExpr(int v, int ln) {
	super(ln);
	value = v;
    }


    // AST printing
    public String toString(String indent) {
	return Integer.toString(value);
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// yields an int
	return new ResolvedIntType();
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// return the int value
	return new IntValue(value);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// return the result value
	return new ILIntConstantExpr(value);
    }
}
