/** The AST node for not-equals ('!=') binary operator expressions. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class NotEqualExpr extends EqualityBinopExpr {
    // Constructor
    public NotEqualExpr(Expr a1, Expr a2, int ln) {
	super(a1, a2, ln);
    }


    // AST printing

    /** return the printable name of this operator */
    protected String opString() {
	return "!=";
    }


    // Evaluation operations

    /** Evaluate the boolean version of this operation */
    protected boolean evaluateBooleanOperation(boolean arg1_value, 
					       boolean arg2_value) {
	return arg1_value != arg2_value;
    }

    /** Evaluate the integer version of this operation */
    protected boolean evaluateIntOperation(int arg1_value, 
					   int arg2_value) {
	return arg1_value != arg2_value;
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Return the boolean version of this operation */
    protected ILExpr loweredBooleanOperation(ILExpr arg1_expr, 
					     ILExpr arg2_expr) {
	// booleans are represented as ints, so use the int comparison
	// operator
	return new ILIntNotEqualExpr(arg1_expr, arg2_expr);
    }

    /** Return the integer version of this operation */
    protected ILExpr loweredIntOperation(ILExpr arg1_expr, 
					 ILExpr arg2_expr) {
	return new ILIntNotEqualExpr(arg1_expr, arg2_expr);
    }
}
