/** The AST node for the null literal (a.k.a. constant) expression. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public class NullLiteralExpr extends Expr {
    // Constructor
    public NullLiteralExpr(int ln) {
	super(ln);
    }


    // AST printing
    public String toString(String indent) {
	return "null";
    }


    // Typechecking operations

    /** Typecheck this expression in the given enclosing context,
     * returning the type of the result */
    protected ResolvedType typecheckHelper(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// yields the special null type, which is a subtype of any
	// reference type
	return new ResolvedNullType();
    }


    // Evaluation operations

    /** Evaluate this expression in the given environment, returning
     * the value of the result */
    public Value evaluate(CodeEnvironment env) throws EvalCompilerException {
	// return the null value
	return new NullValue();
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with code to
     * evaluate this expression, returning an IL expression for
     * computing the result */
    public ILExpr lower(ILFunDecl il_fun_decl) {
	// null is represented by the integer 0
	return new ILIntConstantExpr(0);
    }
}
