/** The AST node for a 'System.out.println' call statement. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;
import java.util.*;

public class PrintlnStmt extends Stmt {
    // Instance variables

    // AST subtrees:
    protected Expr arg;


    // Constructor
    public PrintlnStmt(Expr a, int ln) {
	super(ln);
	arg = a;
    }


    // AST printing
    public String toString(String indent) {
	return "System.out.println(" + arg.toString(indent) + ");";
    }


    // Typechecking operations

    /** Typecheck this statement in the given enclosing context */
    public void typecheck(CodeSymbolTable st)
		throws TypecheckCompilerException {
	// typecheck the argument expression
	ResolvedType arg_type = arg.typecheck(st);

	// verify that it's an integer
	arg_type.checkIsIntType("System.out.println argument", 
				line_number);
    }


    // Evaluation operations

    /** Evaluate this statement in the given environment */
    public void evaluate(CodeEnvironment env) throws EvalCompilerException {
	// evaluate the argument expression
	Value arg_value = arg.evaluate(env);

	// print out the int argument
	System.out.println(arg_value.getIntValue(line_number));
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code
     * for this statement */
    public void lower(ILFunDecl il_fun_decl) {
	// construct the argument subexpression
	ILExpr arg_expr = arg.lower(il_fun_decl);

	// construct a call to the runtime library's print function
	String fun_name = "println_int";
	List<ILExpr> arg_exprs = new LinkedList<ILExpr>();
	arg_exprs.add(arg_expr);
	ILType ignored_result_type = new ResolvedIntType().lower();
	ILExpr call_expr = new ILRuntimeCallExpr(ignored_result_type,
						 fun_name, arg_exprs);

	// generate code to evaluate this call, ignoring the result
	il_fun_decl.addStmt(new ILExprStmt(call_expr));
    }
}
