/** The abstract superclass of all statement AST nodes. */

package AST;

import Typechecker.*;
import Evaluator.*;
import IL.*;
import Helpers.*;

public abstract class Stmt extends ASTNode {
    // Constructor
    public Stmt(int ln) {
	super(ln);
    }


    // Typechecking operations

    /** Typecheck this statement in the given enclosing context */
    public abstract void typecheck(CodeSymbolTable st) 
	throws TypecheckCompilerException;


    // Evaluation operations

    /** Evaluate this statement in the given environment */
    public abstract void evaluate(CodeEnvironment env) 
	throws EvalCompilerException;


    // Lowering (i.e., intermediate-language generation) operations

    /** Augment an intermediate-language function with lowered code
     * for this statement */
    public abstract void lower(ILFunDecl il_fun_decl);
}
