/** The abstract superclass of all AST nodes for types. */

package AST;

import Typechecker.*;
import IL.*;
import Helpers.*;

public abstract class Type extends ASTNode {
    // The resolved type, set after typechecking
    protected ResolvedType resolved_type;

    // Constructor
    public Type(int ln) {
	super(ln);
    }


    // Typechecking operations

    /** Check that this type is a legal type, and if so return the
     * corresponding resolved type to use during typechecking. */
    public ResolvedType resolve(SymbolTable st)
	    throws TypecheckCompilerException {
	if (resolved_type == null) {
	    // needs to be computed
	    resolved_type = computeResolvedType(st);
	}
	return resolved_type;
    }

    /** Return the resolved version of this type, after typechecking. */
    public ResolvedType resolvedType() {
	if (resolved_type == null) {
	    throw new InternalCompilerError(
		    "getting the resolved type before typechecking");
	}
	return resolved_type;
    }

    /** Check that this type is a legal type, and if so return
     * corresponding resolved type to use during typechecking.
     * implemented by subclasses. */
    protected abstract ResolvedType computeResolvedType(SymbolTable st)
	throws TypecheckCompilerException;


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	return resolved_type.lower();
    }
}
