/** This is the concrete class for the location representing the
 * results of comparison operations, i.e., that the condition codes
 * have been computed, and that the given test is intended to be
 * used by the consuming branch */

package CodeGen.X86;

import CodeGen.*;
import Helpers.*;

public class X86ComparisonResultLocation extends Location {
    /** The comparison operator */
    String compare_op;

    /** Constructor */
    public X86ComparisonResultLocation(String op) {
	compare_op = op; 
    }

    /** Return a printable representation of this location */
    public String toString() {
	return "<" + compare_op + ">";
    }

    /** Return the branch code that should be used if testing whether
     * this comparison result is true */
    public String branchTrueOp() {
	return compare_op;
    }	

    /** Return the branch code that should be used if testing whether
     * this comparison result is false */
    public String branchFalseOp() {
	return negatedCompareOp();
    }

    /** Return a new condition code location for the negated test */
    public X86ComparisonResultLocation negate() {
	return new X86ComparisonResultLocation(negatedCompareOp());
    }

    /** Return the negated comparison operator */
    protected String negatedCompareOp() {
	// negate the sense of the comparison outcome
	// signed int tests:
	if (compare_op.equals("l")) return "ge";
	if (compare_op.equals("le")) return "g";
	if (compare_op.equals("ge")) return "l";
	if (compare_op.equals("g")) return "le";
	// unsigned/double tests:
	if (compare_op.equals("b")) return "ae";
	if (compare_op.equals("be")) return "a";
	if (compare_op.equals("ae")) return "b";
	if (compare_op.equals("a")) return "be";
	// equality/disequality tests:
	if (compare_op.equals("e")) return "ne";
	if (compare_op.equals("ne")) return "e";
	throw new InternalCompilerError("unexpected condition");
    }	
}
