/** This is the concrete class for registers on the X86 target
 * machine. */

package CodeGen.X86;

import CodeGen.*;

public class X86Register extends Location {
    /** The register name */
    protected String reg_name;

    /** Whether this register can be allocated */
    protected boolean is_allocatable = false;

    /** Whether this register is allocated at the current point during
     * code generation */
    protected boolean is_allocated = false;


    /** Constructor */
    public X86Register(String rn) {
	reg_name = rn;
    }


    /** Return a printable representation of this location */
    public String toString() {
	return reg_name;
    }


    /** Return the name of the register */
    public String getName() {
	return reg_name;
    }


    /** Return whether this register is allocatable */
    public boolean isAllocatable() {
	return is_allocatable;
    }

    /** Mark this register as allocatable */
    public void setAllocatable() {
	is_allocatable = true;
    }

    /** Return whether this register is currently allocated, if
     * allocatable */
    public boolean isAllocated() {
	return is_allocated;
    }

    /** Mark this register as allocated */
    public void allocate() {
	is_allocated = true;
    }

    /** Mark this register as deallocated */
    public void deallocate() {
	is_allocated = false;
    }
}
