/** The class for class instance values computed during evaluation. */

package Evaluator;

import Typechecker.*;
import Helpers.*;
import java.util.*;

public class ClassValue extends Value {
    // the environment of the class that this is an instance of
    protected ClassEnvironment class_env;

    // the values of the instance's instance variables
    protected Map<String,Value> instance_variable_values = 
	new HashMap<String,Value>();

    // Constructor
    public ClassValue(ClassEnvironment ce) {
	class_env = ce;
    }

    // Printing operations
    public String toString() { 
	// use hashCode() to get a unique identity for this instance
	return "<" + class_env.getClassName() + "#" + hashCode() + ">";
    }
    
    // Evaluation operations

    /** Declare and initialize an instance variable */
    public void initializeInstanceVariable(String name, ResolvedType type) {
	if (! instance_variable_values.containsKey(name)) {
	    // not already initialized; good
	    instance_variable_values.put(name, type.defaultValue());
	} else {
	    throw new InternalCompilerError(
		"reinitializing instance variable named \"" + name + "\"");
	}
    }

    /** If the value is a class instance, return the contents of one
     * of its instance variables, otherwise report an internal
     * error */
    public Value lookupInstanceVariable(String name, int line_number) {
	Value v = instance_variable_values.get(name);
	if (v == null) {
	    // instance variable not declared
	    throw new InternalCompilerError(
		"accessing undefined instance variable named \"" + name + "\"",
		line_number);
	} else {
	    // return its value
	    return v;
	}
    }

    /** If the value is a class instance, assign to one of its
     * instance variables, otherwise report an internal error */
    public void assignInstanceVariable(String name, Value value,
				       int line_number) {
	Value old = instance_variable_values.get(name);
	if (old == null) {
	    // instance variable not declared
	    throw new InternalCompilerError(
		"assigning to undefined instance variable named \"" + 
		name + "\"", line_number);
	} else {
	    // update its value
	    instance_variable_values.put(name, value);
	}
    }

    /** look up the method with the given name in the receiver
     * instance, throwing an evaluation error if the receiver is null.
     * return the method declaration that is invoked. */
    public MethodDeclAndEnv lookupMethod(String name, int line_number)
		throws EvalCompilerException {
	// the receiver is a class type, so do the look-up in the
	// class's interface
	return class_env.lookupMethod(name, line_number);
    }
}
