/** The abstract superclass for environments, which store information
 * during evaluation about a particular scope */

package Evaluator;

import Helpers.*;

public abstract class Environment {
    // Constructor
    public Environment() { }


    // Accessors

    /** Return the environment for the class named 'class_name'. */
    public abstract ClassEnvironment lookupClass(String name, int line_number);

    /** look up the current contents of a local variable */
    public abstract Value lookupLocalVariable(String name, int line_number)
	throws EvalCompilerException;

    /** assign to a local variable */
    public abstract void assignLocalVariable(String name, Value value, 
					     int line_number);


    // Tracing operations

    /** whether to print information about calls and returns */
    public abstract boolean printCalls();

    /** return the number of callers of this method environment (0 =
     * the main class's main method) */
    public int caller_depth() {
	// this should only end up being called on a CodeEnvironment
	throw new InternalCompilerError("shouldn't be here");
    }
}
