/** The class for the environment for the global scope. */

package Evaluator;

import Helpers.*;
import java.util.*;

public class GlobalEnvironment extends Environment {
    /** A map from class names to the environments for each of the
     * top-level classes in the program */
    protected Map<String,ClassEnvironment> classes;

    /** whether to print information about calls and returns during
     * evaluation */
    protected boolean printCalls = false;

    /** constructor */
    public GlobalEnvironment(boolean pc) { 
	printCalls = pc;
	classes = new HashMap<String,ClassEnvironment>();
    }

    /** register a new class environment. */
    public void declareClass(String class_name, ClassEnvironment class_env, 
			     int line_number) {
	if (classes.containsKey(class_name)) {
	    // we've already declared a class with this same name
	    throw new InternalCompilerError(
			"redeclaring class named \"" + class_name + "\"",
			line_number);
	} else {
	    // add the class's environment to the classes map
	    classes.put(class_name, class_env);
	}
    }

    /** Return the environment for the class named 'class_name'. */
    public ClassEnvironment lookupClass(String name, int line_number) {
	ClassEnvironment env = classes.get(name);
	if (env == null) {
	    // class not found
	    throw new InternalCompilerError(
		    "referencing undefined class \"" + name + "\"",
		    line_number);
	} else {
	    // class found
	    return env;
	}
    }

    /** look up the current contents of a local variable */
    public Value lookupLocalVariable(String name, int line_number)
		throws EvalCompilerException {
	// shouldn't be looking for local variables in the global
	// environment, since the ClassEnvironment should have
	// reported the undefined variable error
	throw new InternalCompilerError(
	    "shouldn't be searching the global environment for " +
	    "local variable \"" + name + "\"", line_number);
    }

    /** assign to a local variable */
    public void assignLocalVariable(String name, Value value, 
				    int line_number) {
	// shouldn't be looking for local variables in the global
	// environment, since the ClassEnvironment should have
	// reported the undefined variable error
	throw new InternalCompilerError(
	    "shouldn't be searching the global environment for " +
	    "local variable \"" + name + "\"", line_number);
    }


    // Tracing operations

    /** whether to print information about calls and returns */
    public boolean printCalls() {
	return printCalls;
    }
}
