/** The class for an environment for a scope that's lexically nested
 * within some other scope. */

package Evaluator;

import Helpers.*;

public abstract class NestedEnvironment extends Environment {
    /** the environment for the lexically enclosing scope */
    protected Environment enclosing_env;

    public NestedEnvironment(Environment e) { 
	enclosing_env = e;
    }


    // Accessors

    /** Return the environment for the class named 'class_name'. */
    public ClassEnvironment lookupClass(String name, int line_number) {
	// look in the lexically enclosing scope's environment
	return enclosing_env.lookupClass(name, line_number);
    }


    // Tracing operations

    /** whether to print information about calls and returns */
    public boolean printCalls() {
	// stored in the outermost environment
	return enclosing_env.printCalls();
    }

    /** return the number of callers of this method environment (0 =
     * the main class's main method) */
    public int caller_depth() {
	// skip out to the enclosing method environment
	return enclosing_env.caller_depth();
    }
}
