/** This class is used to throw errors that correspond to unexpected
 * situations in the compiler, i.e., bugs.  It's a subclass of Error,
 * which means that it doesn't have to be listed explicitly in a
 * method's throws clause. */

package Helpers;

public class InternalCompilerError extends Error {
    /** A standard interface for reporting an error */
    public InternalCompilerError(String message) {
	super("Internal compiler error: " + message);
    }

    /** An interface to use if the line of the input where the error
     * occurs is known */
    public InternalCompilerError(String message, int line) {
	super("line " + line + ": Internal compiler error: " + message);
    }

    /** An interface to use if the line and column of the input where
     * the error occurs is known */
    public InternalCompilerError(String message, int line, int column) {
	super("line " + line + ", column " + column + 
	      ": Internal compiler error: " + message);
    }
}
