/** This is the class of IL (intermediate language) expressions that
 * are memory allocations.  The allocated memory is not initialized. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILAllocateExpr extends ILExpr {
    // Instance variable

    // the type of the thing allocated; the result is a pointer to an
    // value of this type.  for instances of this class, the type
    // should be of statically known size, i.e., not include a
    // sequence.  (the ILAllocateSequenceExpr subclass allows
    // allocations of types that end in a sequence.)
    protected ILType type; 


    // Constructor
    public ILAllocateExpr(ILType t) {
	type = t;
    }


    // Printing
    public String toString() {
	return "alloc " + type.toString();
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitAllocate(type);
    }
}
