/** This is the class of IL (intermediate language) expressions that
 * are memory allocations of types that end with a sequence; the
 * number of sequence elements to allocate is given as an extra
 * argument.  The allocated memory is automatically initialized to all
 * zeros. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILAllocateSequenceExpr extends ILAllocateExpr {
    // Instance variable

    // the number of elements of the sequence to allocate space for
    protected ILExpr length;


    // Constructor
    public ILAllocateSequenceExpr(ILType t, ILExpr l) {
	super(t);
	length = l;
    }


    // Printing
    public String toString() {
	return super.toString() + "<" + length.toString() + ">";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitAllocateSequence(type, length);
    }
}
