/** The class for the IL type of code values.  Used only as the target
 * of a pointer, for values that are the addresses of functions or
 * labels */

package IL;

import CodeGen.*;
import Helpers.*;

public class ILCodeType extends ILType {
    // Constructor
    public ILCodeType() { }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "<code>"; 
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// nothing to do
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	// code values are never manipulated as values directly, only
	// via pointer
	throw new InternalCompilerError(
	    "shouldn't be asking for the size of a piece of code");
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	// code values are never manipulated as values directly, only
	// via pointer
	throw new InternalCompilerError(
	    "shouldn't be asking for the alignment of a piece of code");
    }
}
