/** This is the class of IL (intermediate language) expressions that
 * are references to a field of a pointer to a record, either on the
 * left- or right-hand-sides of assignments. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILFieldAccessExpr extends ILAssignableExpr {
    // Instance variables
    protected ILExpr record_ptr;  // the expression yielding a ptr to a record
    protected ILType record_type; // the type of the record
    protected ILRecordMember record_member; 
                                  // the member of the record type to access


    // Constructor
    // (the type of rv should be a pointer to rt, which contains a member rm)
    public ILFieldAccessExpr(ILExpr rv, ILType rt, ILRecordMember rm) {
	record_ptr = rv;
	record_type = rt;
	record_member = rm;
    }


    // Printing
    public String toString() {
	return record_ptr.toString() + "->" + 
	    record_member.getName() + ":" + record_member.getType().toString();
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	return target.emitFieldRead(record_ptr, record_member);
    }

    /** generate target code to assign the argument to this
     * left-hand-side expression, given the set of scratch registers
     * available & allocated. */
    public void codegenAssign(Location rhs_location, Target target)
		throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	target.emitFieldAssign(record_ptr, record_member, rhs_location);
    }
}
