/** This is the class of IL (intermediate language) expressions that
 * are indirect calls through a pointer to a function. */

package IL;

import CodeGen.*;
import java.io.*;
import java.util.*;

public class ILIndirectFunCallExpr extends ILFunCallExpr {
    // Instance variables

    // an expression yielding a pointer to the function to call
    protected ILExpr fun;


    // Constructor
    public ILIndirectFunCallExpr(ILType rt, ILExpr f, List<ILExpr> as) {
	super(rt, as);
	fun = f;
    }


    // Printing
    protected String calleeToString() {
	return "(*" + fun.toString() + ")";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitIndirectCall(result_type, fun, args);
    }
}
