/** This is the class of IL (intermediate language) expressions that
 * are integer greater-than-or-equal comparison operators. (yields 0
 * if false, non-0 if true.) */

package IL;

import CodeGen.*;
import java.io.*;

public class ILIntGreaterThanOrEqualExpr extends ILBinopExpr {
    // Constructor
    public ILIntGreaterThanOrEqualExpr(ILExpr arg1, ILExpr arg2) {
	super(arg1, arg2);
    }


    // Printing
    protected String opToString() {
	return ">=.int";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitIntGreaterThanOrEqualValue(arg1, arg2);
    }

    /** generate target code for this expression, where the result will
     * be used in a conditional branch.  return the location where the
     * result is. */
    public Location codegenTest(Target target) throws IOException {
	return target.emitIntGreaterThanOrEqualTest(arg1, arg2);
    }
}
