/** This is the class of IL (intermediate language) expressions that
 * compute the logical negation of their argument, i.e., if the
 * argument is 0, they return non-zero (e.g. 1), and if the argument
 * is non-zero, they return 0. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILLogicalNegateExpr extends ILUnopExpr {
    // Constructor
    public ILLogicalNegateExpr(ILExpr arg) {
	super(arg);
    }


    // Printing
    protected String opToString() {
	return "not";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitLogicalNegateValue(arg);
    }

    /** generate target code for this expression, where the result will
     * be used in a conditional branch.  return the location where the
     * result is. */
    public Location codegenTest(Target target) throws IOException {
	return target.emitLogicalNegateTest(arg);
    }
}
