/** This is the class of IL (intermediate language) expressions that
 * are references through a pointer, either on the left- or
 * right-hand-sides of assignments. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILPtrAccessExpr extends ILAssignableExpr {
    // Instance variables
    protected ILExpr ptr;   // the pointer expression
    protected ILType type;  // the type of the target to access


    // Constructor
    // (the type of p should be a pointer to t)
    public ILPtrAccessExpr(ILExpr p, ILType t) {
	ptr = p;
	type = t;
    }


    // Printing
    public String toString() {
	return "*(" + ptr.toString() + "):" + type.toString();
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitPtrRead(ptr, type);
    }

    /** generate target code to assign the argument to this
     * left-hand-side expression, given the set of scratch registers
     * available & allocated. */
    public void codegenAssign(Location rhs_location, Target target)
		throws IOException {
	target.emitPtrAssign(ptr, type, rhs_location);
    }
}
