/** The class for IL pointer types. */

package IL;

import CodeGen.*;
import Helpers.*;
import java.util.*;

public class ILPtrType extends ILType {
    // Instance variable

    // the type of the target of the pointer
    protected ILType target_type;


    // Constructor
    public ILPtrType(ILType tt) {
	target_type = tt;
    }


    // Accessor

    // return whether this is a pointer type
    public boolean isPtrType() {
	return true;
    }

    // if a pointer type, then return the type of the pointer target.
    // otherwise report an internal error
    public ILType getPtrTargetType() {
	return target_type;
    }


    // Printing operations

    // return a printable string representation of this type.
    public String toString() { 
	return "*" + target_type.toString();
    }

    // return a printable string representation of this type, but
    // don't show target types below pointer types, as a way to break
    // recursion and keep types from getting too big.  typically
    // called only within ILType toString() methods.
    public String shallowToString() { 
	// don't print out the target type when doing shallow printing
	return "*?";
    }


    // Target code generation operations

    /** compute the layout of this type */
    public void doLayout(Target target) {
	// lay out the target
	target_type.doLayout(target);
    }

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public int byteSize(Target target) {
	return target.ptrByteSize();
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public int byteAlign(Target target) {
	return target.ptrByteAlign();
    }
}
