/** The helper class for members of IL record types. */

package IL;

import Helpers.*;

public class ILRecordMember {
    // Instance variables
    protected ILType type;    // the type of the member
    protected String name;    // the name of the member

    // the byte offset from the start of this record for this member,
    // set during code generation
    protected int byteOffset = -1; // -1 means not set yet


    // Constructor
    public ILRecordMember(ILType t, String n) {
	type = t;
	name = n;
    }


    // Accessors
    public ILType getType() {
	return type;
    }

    public String getName() {
	return name;
    }


    // Printing operations
    public String toString() { 
	return type.shallowToString() + " " + name;
    }


    // Target code generation operations

    /** set the byte offset where this member is found */
    public void setByteOffset(int offset) {
	byteOffset = offset;
    }

    /** return the byte offset where this member is found */
    public int getByteOffset() {
	if (byteOffset == -1) {
	    throw new InternalCompilerError("byte offset not computed yet");
	}
	return byteOffset;
    }
}
