/** This is the class of record initializers for IL (intermediate
 * language) global variables */

package IL;

import CodeGen.*;
import java.io.*;
import java.util.*;

public class ILRecordValue extends ILValue {
    // Instance variable

    // the members of the record value
    protected List<ILRecordValueMember> members;


    // Constructor
    public ILRecordValue(List<ILRecordValueMember> ms) {
	members = ms;
    }


    // Printing operations

    // return a printable string representation of this value
    public String toString() { 
	String s = "{";
	Iterator<ILRecordValueMember> iter = members.iterator();
	boolean first = true;
	while (iter.hasNext()) {
	    ILRecordValueMember member = iter.next();
	    if (first) {
		first = false;
	    } else {
		s += ", ";
	    }
	    s += member.toString();
	}
	s += "}";
	return s;
    }


    // Target code generation operations

    /** generate code for this initializer */
    public void codegenInitializer(Target target) throws IOException {
	// generate initializers for each of the members, in sequence
	Iterator<ILRecordValueMember> iter = members.iterator();
	boolean first = true;
	while (iter.hasNext()) {
	    ILRecordValueMember member = iter.next();
	    // the alignment of the first field is OK, but the
	    // alignment of the later fields may need to be fixed up
	    member.codegenInitializer(target, first);
	    if (first) {
		first = false;
	    }
	}
    }
}
