/** This is the class of IL (intermediate language) expressions that
 * are references to a sequence field of a pointer to a record, either
 * on the left- or right-hand-sides of assignments.  the index of the
 * element to be accessed is given by an extra argument. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILSequenceFieldAccessExpr extends ILFieldAccessExpr {
    // Instance variable
    protected ILExpr index;  // the index of the element of the
			     // sequence to access (origin 0)


    // Constructor
    // (the type of rv should be a pointer to rt, which contains a
    // member rm whose type is a sequence)
    public ILSequenceFieldAccessExpr(ILExpr rv, ILType rt, ILRecordMember rm, 
				     ILExpr i) {
	super(rv, rt, rm);
	index = i;
    }


    // Printing
    public String toString() {
	return super.toString() + "<" + index.toString() + ">";
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	return target.emitSequenceFieldRead(record_ptr, record_member, index);
    }

    /** generate target code to assign the argument to this
     * left-hand-side expression, given the set of scratch registers
     * available & allocated. */
    public void codegenAssign(Location rhs_location, Target target)
		throws IOException {
	// make sure the record has been layed out
	record_type.doLayout(target);
	// generate the code
	target.emitSequenceFieldAssign(record_ptr, record_member, index,
				       rhs_location);
    }
}
