/** This is the class of sequence initializers for IL (intermediate
 * language) global variables */

package IL;

import CodeGen.*;
import java.io.*;
import java.util.*;

public class ILSequenceValue extends ILValue {
    // Instance variables

    // the elements of the sequence value
    protected List<ILValue> elements;

    // the type of each of the elements
    protected ILType elem_type;


    // Constructor
    public ILSequenceValue(List<ILValue> es, ILType et) {
	elements = es;
	elem_type = et;
    }


    // Printing operations

    // return a printable string representation of this value
    public String toString() { 
	String s = "<";
	Iterator<ILValue> iter = elements.iterator();
	boolean first = true;
	while (iter.hasNext()) {
	    ILValue element = iter.next();
	    if (first) {
		first = false;
	    } else {
		s += ", ";
	    }
	    s += element.toString();
	}
	s += ">";
	return s;
    }


    // Target code generation operations

    /** generate code for this initializer */
    public void codegenInitializer(Target target) throws IOException {
	// generate initializers for each of the elements, in sequence
	Iterator<ILValue> iter = elements.iterator();
	boolean first = true;
	while (iter.hasNext()) {
	    ILValue element = iter.next();
	    // the alignment of the first element is OK, but the
	    // alignment of the later elements may need to be fixed up
	    if (first) {
		first = false;
	    } else {
		target.emitAlignDirective(elem_type);
	    }
	    element.codegenInitializer(target);
	}
    }
}
