/** This is the abstract superclass for all IL (intermediate language)
 * types */

package IL;

import Typechecker.*;
import CodeGen.*;
import Helpers.*;
import java.util.*;

public abstract class ILType {
    // Accessors

    // return whether this is the int type
    public boolean isIntType() {
	// by default, it's not
	return false;
    }

    // return whether this is the double type
    public boolean isDoubleType() {
	// by default, it's not
	return false;
    }

    // return whether this is a pointer type
    public boolean isPtrType() {
	// by default, it's not
	return false;
    }

    // if a pointer type, then return the type of the pointer target.
    // otherwise report an internal error
    public ILType getPtrTargetType() {
	throw new InternalCompilerError("expecting a pointer type");
    }

    // if a sequence type, then return the type of the sequence elements.
    // otherwise report an internal error
    public ILType getSequenceElemType() {
	throw new InternalCompilerError("expecting a sequence type");
    }

    // if a record type, then return the member with the given name,
    // otherwise report an internal error
    public ILRecordMember getRecordMember(String member_name) {
	throw new InternalCompilerError("expecting a record type");
    }


    // Printing operations

    // return a printable string representation of this type.
    public abstract String toString();

    // return a printable string representation of this type, but
    // don't show target types below pointer types, as a way to break
    // recursion and keep types from getting too big.  typically
    // called only within ILType toString() methods.
    public String shallowToString() {
	// by default, not a pointer type, so treat non-shallowly
	return toString();
    }


    // Target code generation operations

    /** compute the layout of this type */
    public abstract void doLayout(Target target);

    /** return the size in bytes of a value of this type on the given
     * target machine */
    public abstract int byteSize(Target target);

    /** return the size in bytes of each sequence element, assuming this
     * type ends with a sequence type.  report an error if not. */
    public int sequenceElementByteSize(Target target) {
	// by default, doesn't end with a sequence
	throw new InternalCompilerError("not a type ending in a sequence");
    }

    /** return the alignment constraint in bytes of a value of this
     * type on the given target machine, i.e., at what multiple of
     * bytes must a value of this type begin on */
    public abstract int byteAlign(Target target);
}
