/** This is the class of IL (intermediate language) expressions that
 * are references to local variables, either on the left- or
 * right-hand-sides of assignments. */

package IL;

import CodeGen.*;
import java.io.*;

public class ILVarExpr extends ILAssignableExpr {
    // Instance variable
    protected ILVar var;  // the local variable being accessed

    // Constructor
    public ILVarExpr(ILVar v) {
	var = v;
    }

    // Printing
    public String toString() {
	return var.toString();
    }


    // Target code generation operations

    /** generate target code for this expression, given the set of
     * scratch registers available & allocated. return the location
     * where the result is. */
    public Location codegen(Target target) throws IOException {
	return target.emitVarRead(var.getDecl());
    }

    /** generate target code to assign the argument to this
     * left-hand-side expression, given the set of scratch registers
     * available & allocated. */
    public void codegenAssign(Location rhs_location, Target target)
		throws IOException {
	target.emitVarAssign(var.getDecl(), rhs_location);
    }
}
