/** The abstract superclass for information about class variables. */

package Typechecker;

import Evaluator.*;
import IL.*;
import Helpers.*;
import java.util.*;

public abstract class ClassVarInterface extends VarInterface {
    // the class in which the class variable is declared
    protected ClassSymbolTable class_st;


    // Constructor
    public ClassVarInterface(String name, ResolvedType type, 
			     ClassSymbolTable cst) {
	super(name, type);
	class_st = cst;
    }


    // Evaluation operations

    /** Initialize this class variable in the given instance, if this
     * is an instance variable */
    public abstract void initializeInstanceVariable(ClassValue instance);


    // Lowering operations

    /** if this is an instance variable, generate code into
     * il_fun_decl to initialize this instance variable of
     * instance_var to its default value */
    public abstract void generateInstanceVariableInit(ILVar instance_var,
						      ILFunDecl il_fun_decl);

    /** if this is an instance variable, add a declaration for this
     * variable's member into the members list of the class instance
     * record */
    public abstract void declareInstanceRecordMember(
	List<ILRecordMember> members);
}
