/** The class for information about local variables. */

package Typechecker;

import Evaluator.*;
import IL.*;
import Helpers.*;

public class LocalVarInterface extends VarInterface {
    // Constructor
    public LocalVarInterface(String name, ResolvedType type) {
	super(name, type);
    }


    // Evaluation operations

    /** Return the current contents of this variable */
    public Value lookupVariable(CodeEnvironment env, int line_number) 
		throws EvalCompilerException {
	return env.lookupLocalVariable(name, line_number);
    }

    /** Update the current contents of this variable */
    public void assignVariable(CodeEnvironment env, Value value,
				int line_number) {
	env.assignLocalVariable(name, value, line_number);
    }


    // Lowering (i.e., intermediate-language generation) operations

    /** generate intermediate-language code to assign to this
     * variable */
    public void generateAssignment(ILExpr rhs_expr, ILFunDecl il_fun_decl) {
	// get the IL var for this local variable declaration
	ILVar lhs_var = il_fun_decl.lookupVar(this);

	// generate the assignment
	il_fun_decl.addStmt(new ILAssignStmt(new ILVarExpr(lhs_var), 
					     rhs_expr));
    }

    /** generate intermediate-language code to read from this
     * variable, returning an IL variable holding the result */
    public ILExpr generateRead(ILFunDecl il_fun_decl) {
	// get the IL var for this local variable declaration
	ILVar var = il_fun_decl.lookupVar(this);

	// return a read of it
	return new ILVarExpr(var);
    }
}
