/** The class for (resolved) boolean types */

package Typechecker;

import Evaluator.*;
import IL.*;
import Helpers.*;

public class ResolvedBooleanType extends ResolvedType {
    // Constructor
    public ResolvedBooleanType() { }


    // Printing operations
    public String toString() { 
	return "boolean"; 
    }


    // Testers

    /** return whether this type is a boolean type */
    public boolean isBooleanType() {
	return true;
    }


    // Type comparison operations

    /** Return whether two types should be considered the same type */
    public boolean isSameType(ResolvedType other) {
	return other instanceof ResolvedBooleanType;
    }


    // Evaluation operations

    /** Return the default initial value for a variable of this type */
    public Value defaultValue() {
	// false is the default value
	return new BooleanValue(false);
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	// booleans are represented by ints in the lowered language
	return new ILIntType();
    }

    /** Return an IL expression representing the default value of this
     * type */
    public ILExpr defaultILExpr() {
	// false is the default value, which is represented by the
	// integer 0
	return new ILIntConstantExpr(0);
    }

    /** Return an IL value for initializing a global declaration
     * representing the default value of this type */
    public ILValue defaultILValue() {
	// false is the default value, which is represented by the
	// integer 0
	return new ILIntValue(0);
    }
}
