/** The class for (resolved) int types. */

package Typechecker;

import Evaluator.*;
import IL.*;
import Helpers.*;

public class ResolvedIntType extends ResolvedType {
    // Constructor
    public ResolvedIntType() { }


    // Printing operations
    public String toString() { 
	return "int"; 
    }


    // Testers

    /** return whether this type is an int type */
    public boolean isIntType() {
	return true;
    }


    // Type comparison operations

    /** Return whether two types should be considered the same type */
    public boolean isSameType(ResolvedType other) {
	return other instanceof ResolvedIntType;
    }


    // Evaluation operations

    /** Return the default initial value for a variable of this type */
    public Value defaultValue() {
	// 0 is the default value
	return new IntValue(0);
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	return new ILIntType();
    }

    /** Return an IL expression representing the default value of this
     * type */
    public ILExpr defaultILExpr() {
	// 0 is the default value
	return new ILIntConstantExpr(0);
    }

    /** Return an IL value for initializing a global declaration
     * representing the default value of this type */
    public ILValue defaultILValue() {
	// 0 is the default value
	return new ILIntValue(0);
    }
}
