/** The class for the special (resolved) null type, which is a subtype
 * of any reference type. */

package Typechecker;

import Evaluator.*;
import IL.*;
import Helpers.*;

public class ResolvedNullType extends ResolvedReferenceType {
    // Constructor
    public ResolvedNullType() { }


    // Printing operations
    public String toString() { 
	return "<null>"; 
    }


    // Type comparison operations

    /** Return whether two types should be considered the same type */
    public boolean isSameType(ResolvedType other) {
	return other instanceof ResolvedNullType;
    }

    /** Return whether the receiver should be considered a subtype of
     * the argument */
    public boolean isSubtype(ResolvedType other) {
	// null is a subtype of any reference type
	return other instanceof ResolvedReferenceType;
    }
    

    // Evaluation operations

    /** Return the default initial value for a variable of this type */
    public Value defaultValue() {
	// only the null literal has null type; not variables
	throw new InternalCompilerError(
	    "shouldn't have a variable of null type");
    }


    // Lowering operations

    /** Return the lowered version of this type */
    public ILType lower() {
	// only the null literal has null type; not variables
	throw new InternalCompilerError("shouldn't be lowering the null type");
    }

    /** Return an IL expression representing the default value of this
     * type */
    public ILExpr defaultILExpr() {
	// only the null literal has null type; not variables
	throw new InternalCompilerError(
	    "shouldn't have a variable of null type");
    }

    /** Return an IL value for initializing a global declaration
     * representing the default value of this type */
    public ILValue defaultILValue() {
	// only the null literal has null type; not variables
	throw new InternalCompilerError(
	    "shouldn't have a variable of null type");
    }
}
