/** The abstract superclass for symbol tables, which store information
 * during typechecking about a particular scope */

package Typechecker;

import Helpers.*;

public abstract class SymbolTable {
    public SymbolTable() { }

    /** Return the symbol table for the class named 'class_name'
     * declared in this or some enclosing scope, or throw a
     * TypecheckCompilerException if the class isn't declared.
     * implemented by subclasses. */
    public abstract ClassSymbolTable lookupClass(String class_name, 
						 int line_number) 
	throws TypecheckCompilerException;

    /** Return the variable interface for the variable named 'name'
     * declared in this or some enclosing scope, or throw a
     * TypecheckCompilerException if the variable isn't declared.
     * implemented by subclasses. */
    public abstract VarInterface lookupVariable(String name,
						int line_number) 
	throws TypecheckCompilerException;

    /** Return a printable string representation of the symbol table
     * information */
    public String toString() {
	return toString("");
    }

    /** Return a printable string representation of the symbol table
     * information, with each line indented by the given prefix. */
    public abstract String toString(String indent);
}
